@@ -41,12 +41,13 @@ which allows some access to data within each specific blob type.
 
 use {
     crate::{
-        code_directory::CodeDirectoryBlob, code_requirement::CodeRequirements,
-        error::AppleCodesignError,
+        code_directory::CodeDirectoryBlob, code_hash::compute_code_hashes,
+        code_requirement::CodeRequirements, error::AppleCodesignError, signing::SigningSettings,
     },
+    cryptographic_message_syntax::time_stamp_message_http,
     goblin::mach::{
-        constants::{SEG_LINKEDIT, SEG_TEXT},
-        load_command::CommandVariant,
+        constants::{SEG_LINKEDIT, SEG_PAGEZERO, SEG_TEXT},
+        load_command::{CommandVariant, LinkeditDataCommand, SIZEOF_LINKEDIT_DATA_COMMAND},
         MachO,
     },
     scroll::{IOwrite, Pread},
@@ -57,6 +58,7 @@ use {
         convert::{TryFrom, TryInto},
         io::Write,
     },
+    x509_certificate::DigestAlgorithm,
 };
 
 /// Defines a typed slot within code signing data.
@@ -1246,31 +1248,274 @@ pub struct Scatter {
     spare: u64,
 }
 
-/// Determine the start and end offset of the executable segment of a binary.
-pub fn find_executable_segment_boundary(macho: &MachO) -> Result<(u64, u64), AppleCodesignError> {
-    // There might be some Mach-O segment flags we can query for this. For now,
-    // we find the offset after the __TEXT segment.
+pub trait AppleSignable {
+    /// Obtain the code signature in the entity.
+    ///
+    /// Returns `Ok(None)` if no signature exists, `Ok(Some)` if it does, or
+    /// `Err` if there is a parse error.
+    fn code_signature(&self) -> Result<Option<EmbeddedSignature>, AppleCodesignError>;
 
-    let segment = macho
-        .segments
-        .iter()
-        .find(|segment| {
-            if let Ok(name) = segment.name() {
-                name == SEG_TEXT
+    /// Determine the start and end offset of the executable segment of a binary.
+    fn executable_segment_boundary(&self) -> Result<(u64, u64), AppleCodesignError>;
+
+    /// The start offset of the code signature data within the __LINKEDIT segment.
+    fn code_signature_linkedit_start_offset(&self) -> Option<u32>;
+
+    /// The end offset of the code signature data within the __LINKEDIT segment.
+    fn code_signature_linkedit_end_offset(&self) -> Option<u32>;
+
+    /// The byte offset within the binary at which point "code" stops.
+    ///
+    /// If a signature is present, this is the offset of the start of the
+    /// signature. Else it represents the end of the binary.
+    fn code_limit_binary_offset(&self) -> Result<u64, AppleCodesignError>;
+
+    /// Obtain __LINKEDIT segment data before the signature data.
+    fn linkedit_data_before_signature(&self) -> Option<&[u8]>;
+
+    /// Obtain slices of segment data suitable for digesting.
+    ///
+    /// The slices are likely digested as part of computing digests
+    /// embedded in the code directory.
+    fn digestable_segment_data(&self) -> Vec<&[u8]>;
+
+    /// Resolve the load command for the code signature.
+    fn code_signature_load_command(&self) -> Option<LinkeditDataCommand>;
+
+    /// Determines whether this crate is capable of signing a given Mach-O binary.
+    ///
+    /// Code in this crate is limited in the amount of Mach-O binary manipulation
+    /// it can perform (supporting rewriting all valid Mach-O binaries effectively
+    /// requires low-level awareness of all Mach-O constructs in order to perform
+    /// offset manipulation). This function can be used to test signing
+    /// compatibility.
+    ///
+    /// We currently only support signing Mach-O files already containing an
+    /// embedded signature. Often linked binaries automatically contain an embedded
+    /// signature containing just the code directory (without a cryptographically
+    /// signed signature), so this limitation hopefully isn't impactful.
+    fn check_signing_capability(&self) -> Result<(), AppleCodesignError>;
+
+    /// Estimate the size in bytes of an embedded code signature.
+    fn estimate_embedded_signature_size(
+        &self,
+        settings: &SigningSettings,
+    ) -> Result<usize, AppleCodesignError>;
+}
+
+impl<'a> AppleSignable for MachO<'a> {
+    fn code_signature(&self) -> Result<Option<EmbeddedSignature>, AppleCodesignError> {
+        if let Some(signature) = find_signature_data(self)? {
+            Ok(Some(EmbeddedSignature::from_bytes(
+                signature.signature_data,
+            )?))
+        } else {
+            Ok(None)
+        }
+    }
+
+    fn executable_segment_boundary(&self) -> Result<(u64, u64), AppleCodesignError> {
+        let segment = self
+            .segments
+            .iter()
+            .find(|segment| matches!(segment.name(), Ok(SEG_TEXT)))
+            .ok_or_else(|| AppleCodesignError::InvalidBinary("no __TEXT segment".into()))?;
+
+        Ok((segment.fileoff, segment.fileoff + segment.data.len() as u64))
+    }
+
+    fn code_signature_linkedit_start_offset(&self) -> Option<u32> {
+        let segment = self
+            .segments
+            .iter()
+            .find(|segment| matches!(segment.name(), Ok(SEG_LINKEDIT)));
+
+        if let (Some(segment), Some(command)) = (segment, self.code_signature_load_command()) {
+            Some((command.dataoff as u64 - segment.fileoff) as u32)
+        } else {
+            None
+        }
+    }
+
+    fn code_signature_linkedit_end_offset(&self) -> Option<u32> {
+        let start_offset = self.code_signature_linkedit_start_offset()?;
+
+        self.code_signature_load_command()
+            .map(|command| start_offset + command.datasize)
+    }
+
+    fn code_limit_binary_offset(&self) -> Result<u64, AppleCodesignError> {
+        let last_segment = self
+            .segments
+            .last()
+            .ok_or(AppleCodesignError::MissingLinkedit)?;
+        if !matches!(last_segment.name(), Ok(SEG_LINKEDIT)) {
+            return Err(AppleCodesignError::LinkeditNotLast);
+        }
+
+        if let Some(offset) = self.code_signature_linkedit_start_offset() {
+            Ok(last_segment.fileoff + offset as u64)
+        } else {
+            Ok(last_segment.fileoff + last_segment.data.len() as u64)
+        }
+    }
+
+    fn linkedit_data_before_signature(&self) -> Option<&[u8]> {
+        let segment = self
+            .segments
+            .iter()
+            .find(|segment| matches!(segment.name(), Ok(SEG_LINKEDIT)));
+
+        if let Some(segment) = segment {
+            if let Some(offset) = self.code_signature_linkedit_start_offset() {
+                Some(&segment.data[0..offset as usize])
             } else {
-                false
+                Some(&segment.data)
+            }
+        } else {
+            None
+        }
+    }
+
+    fn digestable_segment_data(&self) -> Vec<&[u8]> {
+        self.segments
+            .iter()
+            .filter(|segment| !matches!(segment.name(), Ok(SEG_PAGEZERO)))
+            .map(|segment| {
+                if matches!(segment.name(), Ok(SEG_LINKEDIT)) {
+                    self.linkedit_data_before_signature()
+                        .expect("__LINKEDIT data should resolve")
+                } else {
+                    &segment.data
+                }
+            })
+            .collect::<Vec<_>>()
+    }
+
+    fn code_signature_load_command(&self) -> Option<LinkeditDataCommand> {
+        self.load_commands.iter().find_map(|lc| {
+            if let CommandVariant::CodeSignature(command) = lc.command {
+                Some(command)
+            } else {
+                None
             }
         })
-        .ok_or(AppleCodesignError::MissingLinkedit)?;
+    }
 
-    Ok((segment.fileoff, segment.fileoff + segment.data.len() as u64))
+    fn check_signing_capability(&self) -> Result<(), AppleCodesignError> {
+        let last_segment = self
+            .segments
+            .iter()
+            .last()
+            .ok_or(AppleCodesignError::MissingLinkedit)?;
+
+        // Last segment needs to be __LINKEDIT so we don't have to write offsets.
+        if !matches!(last_segment.name(), Ok(SEG_LINKEDIT)) {
+            return Err(AppleCodesignError::LinkeditNotLast);
+        }
+
+        // Rules:
+        //
+        // 1. If there is an existing signature, there must be no data in
+        //    the binary after it. (We don't know how to update references to
+        //    other data to reflect offset changes.)
+        // 2. If there isn't an existing signature, there must be "room" between
+        //    the last load command and the first section to write a new load
+        //    command for the signature.
+
+        if let Some(offset) = self.code_signature_linkedit_end_offset() {
+            if offset as usize == last_segment.data.len() {
+                Ok(())
+            } else {
+                Err(AppleCodesignError::DataAfterSignature)
+            }
+        } else {
+            let last_load_command = self
+                .load_commands
+                .iter()
+                .last()
+                .ok_or_else(|| AppleCodesignError::InvalidBinary("no load commands".into()))?;
+
+            let first_section = self
+                .segments
+                .iter()
+                .map(|segment| segment.sections())
+                .collect::<Result<Vec<_>, _>>()?
+                .into_iter()
+                .flatten()
+                .next()
+                .ok_or_else(|| AppleCodesignError::InvalidBinary("no sections".into()))?;
+
+            let load_commands_end_offset =
+                last_load_command.offset + last_load_command.command.cmdsize();
+
+            if first_section.0.offset as usize - load_commands_end_offset
+                >= SIZEOF_LINKEDIT_DATA_COMMAND
+            {
+                Ok(())
+            } else {
+                Err(AppleCodesignError::LoadCommandNoRoom)
+            }
+        }
+    }
+
+    fn estimate_embedded_signature_size(
+        &self,
+        settings: &SigningSettings,
+    ) -> Result<usize, AppleCodesignError> {
+        // Assume the common data structures are 1024 bytes.
+        let mut size = 1024;
+
+        // Reserve room for the code digests, which are proportional to binary size.
+        // We could avoid doing the actual digesting work here. But until people
+        // complain, don't worry about it.
+        size += compute_code_hashes(self, *settings.digest_type(), None)?
+            .into_iter()
+            .map(|x| x.len())
+            .sum::<usize>();
+
+        // Assume the CMS data will take a fixed size.
+        if settings.signing_key().is_some() {
+            size += 4096;
+        }
+
+        // Long certificate chains could blow up the size. Account for those.
+        for cert in settings.certificate_chain() {
+            size += cert.constructed_data().len();
+        }
+
+        // Obtain an actual timestamp token of placeholder data and use its length.
+        // This may be excessive to actually query the time-stamp server and issue
+        // a token. But these operations should be "cheap."
+        if let Some(timestamp_url) = settings.time_stamp_url() {
+            let message = b"deadbeef".repeat(32);
+
+            if let Ok(response) =
+                time_stamp_message_http(timestamp_url.clone(), &message, DigestAlgorithm::Sha256)
+            {
+                if response.is_success() {
+                    if let Some(l) = response.token_content_size() {
+                        size += l;
+                    } else {
+                        size += 8192;
+                    }
+                } else {
+                    size += 8192;
+                }
+            } else {
+                size += 8192;
+            }
+        }
+
+        // Align on 1k boundaries just because.
+        size += 1024 - size % 1024;
+
+        Ok(size)
+    }
 }
 
 /// Describes signature data embedded within a Mach-O binary.
 pub struct MachOSignatureData<'a> {
-    /// The number of segments in the Mach-O binary.
-    pub segments_count: usize,
-
     /// Which segment offset is the `__LINKEDIT` segment.
     pub linkedit_segment_index: usize,
 
@@ -1342,7 +1587,6 @@ pub fn find_signature_data<'a>(
         let signature_data = &linkedit.data[signature_start_offset..signature_end_offset];
 
         Ok(Some(MachOSignatureData {
-            segments_count: obj.segments.len(),
             linkedit_segment_index,
             linkedit_segment_start_offset,
             linkedit_segment_end_offset,
@@ -1358,23 +1602,6 @@ pub fn find_signature_data<'a>(
     }
 }
 
-/// Parse raw Mach-O signature data into a data structure.
-///
-/// The source data likely came from the `__LINKEDIT` segment and was
-/// discovered via `find_signature_data()`.
-///
-/// Only a high-level parse of the super blob and its blob indices is performed:
-/// the parser does not look inside individual blob payloads.
-pub fn parse_signature_data(data: &[u8]) -> Result<EmbeddedSignature<'_>, AppleCodesignError> {
-    let magic: u32 = data.pread_with(0, scroll::BE)?;
-
-    if magic == u32::from(CodeSigningMagic::EmbeddedSignature) {
-        EmbeddedSignature::from_bytes(data)
-    } else {
-        Err(AppleCodesignError::BadMagic("embedded signature superblob"))
-    }
-}
-
 #[cfg(test)]
 mod tests {
     use {
@@ -1417,12 +1644,8 @@ mod tests {
     fn find_apple_embedded_signature<'a>(
         macho: &'a goblin::mach::MachO,
     ) -> Option<EmbeddedSignature<'a>> {
-        if let Ok(Some(codesign_data)) = find_signature_data(macho) {
-            if let Ok(signature) = parse_signature_data(codesign_data.signature_data) {
-                Some(signature)
-            } else {
-                None
-            }
+        if let Ok(Some(signature)) = macho.code_signature() {
+            Some(signature)
         } else {
             None
         }
