@@ -13,9 +13,9 @@ use {
         code_requirement::{CodeRequirementExpression, CodeRequirements},
         error::AppleCodesignError,
         macho::{
-            create_superblob, find_executable_segment_boundary, find_signature_data,
-            parse_signature_data, Blob, BlobWrapperBlob, CodeSigningMagic, CodeSigningSlot, Digest,
-            DigestType, EmbeddedSignature, EntitlementsBlob, RequirementSetBlob, RequirementType,
+            create_superblob, AppleSignable, Blob, BlobWrapperBlob, CodeSigningMagic,
+            CodeSigningSlot, Digest, DigestType, EmbeddedSignature, EntitlementsBlob,
+            RequirementSetBlob, RequirementType,
         },
         policy::derive_designated_requirements,
         signing::{DesignatedRequirementMode, SettingsScope, SigningSettings},
@@ -25,13 +25,15 @@ use {
     cryptographic_message_syntax::{asn1::rfc5652::OID_ID_DATA, SignedDataBuilder, SignerBuilder},
     goblin::mach::{
         constants::{SEG_LINKEDIT, SEG_PAGEZERO},
-        fat::FAT_MAGIC,
-        fat::{SIZEOF_FAT_ARCH, SIZEOF_FAT_HEADER},
-        load_command::{CommandVariant, LinkeditDataCommand, SegmentCommand32, SegmentCommand64},
+        load_command::{
+            CommandVariant, LinkeditDataCommand, SegmentCommand32, SegmentCommand64,
+            LC_CODE_SIGNATURE, SIZEOF_LINKEDIT_DATA_COMMAND,
+        },
         parse_magic_and_ctx, Mach, MachO,
     },
-    scroll::{ctx::SizeWith, IOwrite, Pwrite},
+    scroll::{ctx::SizeWith, IOwrite},
     std::{borrow::Cow, cmp::Ordering, collections::HashMap, io::Write},
+    tugger_apple::create_universal_macho,
     x509_certificate::{rfc5652::AttributeValue, DigestAlgorithm},
 };
 
@@ -45,37 +47,6 @@ const CDHASH_PLIST_OID: bcder::ConstOid = Oid(&[42, 134, 72, 134, 247, 99, 100,
 /// 1.2.840.113635.100.9.2
 const CDHASH_SHA256_OID: bcder::ConstOid = Oid(&[42, 134, 72, 134, 247, 99, 100, 9, 2]);
 
-/// Determines whether this crate is capable of signing a given Mach-O binary.
-///
-/// Code in this crate is limited in the amount of Mach-O binary manipulation
-/// it can perform (supporting rewriting all valid Mach-O binaries effectively
-/// requires low-level awareness of all Mach-O constructs in order to perform
-/// offset manipulation). This function can be used to test signing
-/// compatibility.
-///
-/// We currently only support signing Mach-O files already containing an
-/// embedded signature. Often linked binaries automatically contain an embedded
-/// signature containing just the code directory (without a cryptographically
-/// signed signature), so this limitation hopefully isn't impactful.
-pub fn check_signing_capability(macho: &MachO) -> Result<(), AppleCodesignError> {
-    match find_signature_data(macho)? {
-        Some(signature) => {
-            // __LINKEDIT needs to be the final segment so we don't have to rewrite
-            // offsets.
-            if signature.linkedit_segment_index != macho.segments.len() - 1 {
-                Err(AppleCodesignError::LinkeditNotLast)
-            // There can be no meaningful data after the signature because we don't
-            // know how to rewrite it.
-            } else if signature.signature_end_offset != signature.linkedit_segment_data.len() {
-                Err(AppleCodesignError::DataAfterSignature)
-            } else {
-                Ok(())
-            }
-        }
-        None => Err(AppleCodesignError::BinaryNoCodeSignature),
-    }
-}
-
 /// Obtain the XML plist containing code directory hashes.
 ///
 /// This plist is embedded as a signed attribute in the CMS signature.
@@ -116,18 +87,18 @@ fn create_macho_with_signature(
     macho: &MachO,
     signature_data: &[u8],
 ) -> Result<Vec<u8>, AppleCodesignError> {
-    let existing_signature =
-        find_signature_data(macho)?.ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
-
     // This should have already been called. But we do it again out of paranoia.
-    check_signing_capability(macho)?;
+    macho.check_signing_capability()?;
 
     // The assumption made by checking_signing_capability() is that signature data
     // is at the end of the __LINKEDIT segment. So the replacement segment is the
     // existing segment truncated at the signature start followed by the new signature
     // data.
-    let new_linkedit_segment_size =
-        existing_signature.signature_start_offset + signature_data.len();
+    let new_linkedit_segment_size = macho
+        .linkedit_data_before_signature()
+        .ok_or(AppleCodesignError::MissingLinkedit)?
+        .len()
+        + signature_data.len();
 
     let mut cursor = std::io::Cursor::new(Vec::<u8>::new());
 
@@ -137,16 +108,29 @@ fn create_macho_with_signature(
         .1
         .expect("context should have been parsed before");
 
-    cursor.iowrite_with(macho.header, ctx)?;
+    // If there isn't a code signature presently, we'll need to introduce a load
+    // command for it.
+    let mut header = macho.header;
+    if macho.code_signature_load_command().is_none() {
+        header.ncmds += 1;
+        header.sizeofcmds += SIZEOF_LINKEDIT_DATA_COMMAND as u32;
+    }
+
+    cursor.iowrite_with(header, ctx)?;
 
     // Following the header are load commands. We need to update load commands
     // to reflect changes to the signature size and __LINKEDIT segment size.
+
+    let mut seen_signature_load_command = false;
+
     for load_command in &macho.load_commands {
         let original_command_data =
             &macho_data[load_command.offset..load_command.offset + load_command.command.cmdsize()];
 
         let written_len = match &load_command.command {
             CommandVariant::CodeSignature(command) => {
+                seen_signature_load_command = true;
+
                 let mut command = *command;
                 command.datasize = signature_data.len() as _;
 
@@ -196,6 +180,18 @@ fn create_macho_with_signature(
         cursor.write_all(&original_command_data[written_len..])?;
     }
 
+    // If we didn't see a signature load command, write one out now.
+    if !seen_signature_load_command {
+        let command = LinkeditDataCommand {
+            cmd: LC_CODE_SIGNATURE,
+            cmdsize: SIZEOF_LINKEDIT_DATA_COMMAND as _,
+            dataoff: macho.code_limit_binary_offset()? as _,
+            datasize: signature_data.len() as _,
+        };
+
+        cursor.iowrite_with(command, ctx.le)?;
+    }
+
     // Write out segments, updating the __LINKEDIT segment when we encounter it.
     for segment in macho.segments.iter() {
         assert!(segment.fileoff == 0 || segment.fileoff == cursor.position());
@@ -210,8 +206,9 @@ fn create_macho_with_signature(
         match segment.name() {
             Ok(SEG_LINKEDIT) => {
                 cursor.write_all(
-                    &existing_signature.linkedit_segment_data
-                        [0..existing_signature.signature_start_offset],
+                    macho
+                        .linkedit_data_before_signature()
+                        .expect("__LINKEDIT segment data should resolve"),
                 )?;
                 cursor.write_all(signature_data)?;
             }
@@ -251,11 +248,8 @@ fn create_macho_with_signature(
 /// can't be known until the Code Directory, CMS blob, and SuperBlob are all
 /// created.
 ///
-/// Our solution to this problem is to create an intermediate Mach-O binary with
-/// placeholder bytes for the signature. We then digest this. When writing
-/// the final Mach-O binary we simply replace NULLs with actual signature data,
-/// leaving any extra at the end, because truncating the file would require
-/// adjusting Mach-O load commands and changing content digests.
+/// Our solution to this problem is to estimate the size of the embedded
+/// signature data and then pad the unused data will 0s.
 #[derive(Debug)]
 pub struct MachOSigner<'data> {
     /// Raw data backing parsed Mach-O binary.
@@ -275,7 +269,7 @@ impl<'data> MachOSigner<'data> {
 
         let machos = match mach {
             Mach::Binary(macho) => {
-                check_signing_capability(&macho)?;
+                macho.check_signing_capability()?;
 
                 vec![macho]
             }
@@ -284,7 +278,7 @@ impl<'data> MachOSigner<'data> {
 
                 for index in 0..multiarch.narches {
                     let macho = multiarch.get(index)?;
-                    check_signing_capability(&macho)?;
+                    macho.check_signing_capability()?;
 
                     machos.push(macho);
                 }
@@ -314,45 +308,37 @@ impl<'data> MachOSigner<'data> {
                 let settings =
                     settings.as_nested_macho_settings(index, original_macho.header.cputype());
 
-                let signature_data = find_signature_data(original_macho)?;
-                let signature = if let Some(data) = &signature_data {
-                    Some(parse_signature_data(&data.signature_data)?)
-                } else {
-                    None
-                };
+                let signature_len = original_macho.estimate_embedded_signature_size(&settings)?;
 
                 // Derive an intermediate Mach-O with placeholder NULLs for signature
-                // data so Code Directory digests are correct.
-                let placeholder_signature_len = self
-                    .create_superblob(&settings, original_macho, signature.as_ref())?
-                    .len();
-                let placeholder_signature = b"\0".repeat(placeholder_signature_len + 1024);
+                // data so Code Directory digests over the load commands are correct.
+                let placeholder_signature_data = b"\0".repeat(signature_len);
 
-                // TODO calling this twice could be undesirable, especially if using
-                // a timestamp server. Should we call in no-op mode or write a size
-                // estimation function instead?
                 let intermediate_macho_data = create_macho_with_signature(
                     self.macho_data(index),
                     original_macho,
-                    &placeholder_signature,
+                    &placeholder_signature_data,
                 )?;
 
                 // A nice side-effect of this is that it catches bugs if we write malformed Mach-O!
                 let intermediate_macho = MachO::parse(&intermediate_macho_data, 0)?;
 
-                let mut signature_data =
-                    self.create_superblob(&settings, &intermediate_macho, signature.as_ref())?;
+                let mut signature_data = self.create_superblob(
+                    &settings,
+                    &intermediate_macho,
+                    original_macho.code_signature()?.as_ref(),
+                )?;
 
                 // The Mach-O writer adjusts load commands based on the signature length. So pad
                 // with NULLs to get to our placeholder length.
-                match signature_data.len().cmp(&placeholder_signature.len()) {
+                match signature_data.len().cmp(&placeholder_signature_data.len()) {
                     Ordering::Greater => {
                         return Err(AppleCodesignError::SignatureDataTooLarge);
                     }
                     Ordering::Equal => {}
                     Ordering::Less => {
                         signature_data.extend_from_slice(
-                            &b"\0".repeat(placeholder_signature.len() - signature_data.len()),
+                            &b"\0".repeat(placeholder_signature_data.len() - signature_data.len()),
                         );
                     }
                 }
@@ -365,50 +351,10 @@ impl<'data> MachOSigner<'data> {
             })
             .collect::<Result<Vec<_>, AppleCodesignError>>()?;
 
-        match Mach::parse(&self.macho_data).expect("should reparse without error") {
-            Mach::Binary(_) => {
-                assert_eq!(binaries.len(), 1);
-                writer.write_all(&binaries[0])?;
-            }
-            Mach::Fat(multiarch) => {
-                assert_eq!(binaries.len(), multiarch.narches);
-
-                // The fat arch header records the start offset and size of each binary.
-                // Do a pass over the binaries and calculate these offsets.
-                //
-                // Binaries appear to be 4k page aligned, so also collect padding
-                // information so we write nulls later.
-                let mut current_offset = SIZEOF_FAT_HEADER + SIZEOF_FAT_ARCH * binaries.len();
-                let mut write_instructions = Vec::with_capacity(binaries.len());
-
-                for (index, arch) in multiarch.iter_arches().enumerate() {
-                    let mut arch = arch?;
-                    let macho_data = &binaries[index];
-
-                    let pad_bytes = 4096 - current_offset % 4096;
-
-                    arch.offset = (current_offset + pad_bytes) as _;
-                    arch.size = macho_data.len() as _;
-
-                    current_offset += macho_data.len() + pad_bytes;
-
-                    write_instructions.push((arch, pad_bytes, macho_data));
-                }
-
-                writer.iowrite_with(FAT_MAGIC, scroll::BE)?;
-                writer.iowrite_with(multiarch.narches as u32, scroll::BE)?;
-
-                for (fat_arch, _, _) in &write_instructions {
-                    let mut buffer = [0u8; SIZEOF_FAT_ARCH];
-                    buffer.pwrite_with(fat_arch, 0, scroll::BE)?;
-                    writer.write_all(&buffer)?;
-                }
-
-                for (_, pad_bytes, macho_data) in write_instructions {
-                    writer.write_all(&b"\0".repeat(pad_bytes))?;
-                    writer.write_all(macho_data)?;
-                }
-            }
+        if binaries.len() > 1 {
+            create_universal_macho(writer, binaries.iter().map(|x| x.as_slice()))?;
+        } else {
+            writer.write_all(&binaries[0])?;
         }
 
         Ok(())
@@ -444,16 +390,16 @@ impl<'data> MachOSigner<'data> {
         &self,
         settings: &SigningSettings,
         macho: &MachO,
-        signature: Option<&EmbeddedSignature>,
+        previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<Vec<u8>, AppleCodesignError> {
-        let code_directory = self.create_code_directory(settings, macho, signature)?;
+        let code_directory = self.create_code_directory(settings, macho, previous_signature)?;
 
         // By convention, the Code Directory goes first.
         let mut blobs = vec![(
             CodeSigningSlot::CodeDirectory,
             code_directory.to_blob_bytes()?,
         )];
-        blobs.extend(self.create_special_blobs(settings, signature)?);
+        blobs.extend(self.create_special_blobs(settings, previous_signature)?);
 
         // And the CMS signature goes last.
         if settings.signing_key().is_some() {
@@ -531,16 +477,16 @@ impl<'data> MachOSigner<'data> {
             signer
         };
 
-        let ber = SignedDataBuilder::default()
+        let der = SignedDataBuilder::default()
             // The default is `signed-data`. But Apple appears to use the `data` content-type,
             // in violation of RFC 5652 Section 5, which says `signed-data` should be
             // used when there are signatures.
             .content_type(Oid(OID_ID_DATA.as_ref().into()))
             .signer(signer)
             .certificates(settings.certificate_chain().iter().cloned())
-            .build_ber()?;
+            .build_der()?;
 
-        Ok(ber)
+        Ok(der)
     }
 
     /// Attempt to resolve the binary identifier to use.
@@ -550,10 +496,10 @@ impl<'data> MachOSigner<'data> {
     fn get_binary_identifier(
         &self,
         settings: &SigningSettings,
-        signature: Option<&EmbeddedSignature>,
+        previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<String, AppleCodesignError> {
         let previous_cd =
-            signature.and_then(|signature| signature.code_directory().unwrap_or(None));
+            previous_signature.and_then(|signature| signature.code_directory().unwrap_or(None));
 
         match settings.binary_identifier(SettingsScope::Main) {
             Some(ident) => Ok(ident.to_string()),
@@ -576,15 +522,13 @@ impl<'data> MachOSigner<'data> {
         &self,
         settings: &SigningSettings,
         macho: &MachO,
-        signature: Option<&EmbeddedSignature>,
+        previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<CodeDirectoryBlob<'static>, AppleCodesignError> {
         // TODO support defining or filling in proper values for fields with
         // static values.
 
         let previous_cd =
-            signature.and_then(|signature| signature.code_directory().unwrap_or(None));
-
-        let signature_data = find_signature_data(macho)?;
+            previous_signature.and_then(|signature| signature.code_directory().unwrap_or(None));
 
         let mut flags = CodeSignatureFlags::empty();
 
@@ -610,52 +554,15 @@ impl<'data> MachOSigner<'data> {
         // Code limit fields hold the file offset at which code digests stop. This
         // is the file offset in the `__LINKEDIT` segment when the embedded signature
         // SuperBlob begins.
-        let (code_limit, code_limit_64) = match &signature_data {
-            Some(sig) => {
-                // If binary already has signature data, take existing signature start offset.
-                let limit = sig.linkedit_signature_start_offset;
-
-                if limit > u32::MAX as usize {
-                    (0, Some(limit as u64))
-                } else {
-                    (limit as u32, None)
-                }
-            }
-            None => {
-                // No existing signature in binary. Look for __LINKEDIT and use its
-                // end offset.
-                match macho
-                    .segments
-                    .iter()
-                    .find(|x| matches!(x.name(), Ok("__LINKEDIT")))
-                {
-                    Some(segment) => {
-                        let limit = segment.fileoff as usize + segment.data.len();
-
-                        if limit > u32::MAX as usize {
-                            (0, Some(limit as u64))
-                        } else {
-                            (limit as u32, None)
-                        }
-                    }
-                    None => {
-                        let last_segment = macho.segments.iter().last().unwrap();
-                        let limit = last_segment.fileoff as usize + last_segment.data.len();
-
-                        if limit > u32::MAX as usize {
-                            (0, Some(limit as u64))
-                        } else {
-                            (limit as u32, None)
-                        }
-                    }
-                }
-            }
+        let (code_limit, code_limit_64) = match macho.code_limit_binary_offset()? {
+            x if x > u32::MAX as u64 => (0, Some(x)),
+            x => (x as u32, None),
         };
 
         let platform = 0;
         let page_size = 4096u32;
 
-        let (exec_seg_base, exec_seg_limit) = find_executable_segment_boundary(macho)?;
+        let (exec_seg_base, exec_seg_limit) = macho.executable_segment_boundary()?;
         let (exec_seg_base, exec_seg_limit) = (Some(exec_seg_base), Some(exec_seg_limit));
 
         let mut exec_seg_flags = None;
@@ -685,7 +592,7 @@ impl<'data> MachOSigner<'data> {
                 .collect::<Vec<_>>();
 
         let mut special_hashes = self
-            .create_special_blobs(settings, signature)?
+            .create_special_blobs(settings, previous_signature)?
             .into_iter()
             .map(|(slot, data)| {
                 Ok((
@@ -741,17 +648,16 @@ impl<'data> MachOSigner<'data> {
             }
         }
 
-        let ident = Cow::Owned(self.get_binary_identifier(settings, signature)?);
+        let ident = Cow::Owned(self.get_binary_identifier(settings, previous_signature)?);
 
         let team_name = match settings.team_id() {
             Some(team_name) => Some(Cow::Owned(team_name.to_string())),
             None => {
                 if let Some(previous_cd) = &previous_cd {
-                    if let Some(name) = &previous_cd.team_name {
-                        Some(Cow::Owned(name.clone().into_owned()))
-                    } else {
-                        None
-                    }
+                    previous_cd
+                        .team_name
+                        .as_ref()
+                        .map(|name| Cow::Owned(name.clone().into_owned()))
                 } else {
                     None
                 }
@@ -802,7 +708,7 @@ impl<'data> MachOSigner<'data> {
     pub fn create_special_blobs(
         &self,
         settings: &SigningSettings,
-        signature: Option<&EmbeddedSignature>,
+        previous_signature: Option<&EmbeddedSignature>,
     ) -> Result<Vec<(CodeSigningSlot, Vec<u8>)>, AppleCodesignError> {
         let mut res = Vec::new();
 
@@ -813,7 +719,8 @@ impl<'data> MachOSigner<'data> {
                 // If we are using an Apple-issued cert, this should automatically
                 // derive appropriate designated requirements.
                 if let Some((_, cert)) = settings.signing_key() {
-                    let identifier = Some(self.get_binary_identifier(settings, signature)?);
+                    let identifier =
+                        Some(self.get_binary_identifier(settings, previous_signature)?);
 
                     if let Some(expr) = derive_designated_requirements(cert, identifier)? {
                         requirements.push(expr);
