@@ -44,7 +44,7 @@ use {
         code_requirement::CodeRequirements,
         error::AppleCodesignError,
         macho::{
-            find_signature_data, parse_signature_data, Blob, CodeSigningSlot, DigestType,
+            find_signature_data, AppleSignable, Blob, CodeSigningSlot, DigestType,
             RequirementSetBlob,
         },
         macho_signing::MachOSigner,
@@ -72,16 +72,14 @@ Possible values are:
 
 blobs
    Low-level information on the records in the embedded code signature.
-cms-ber
-   BER encoded ASN.1 of the CMS SignedObject message containing a
-   cryptographic signature over content. (This will print binary
-   to stdout.)
+cms-info
+   Print important information about the CMS data structure.
 cms-pem
-   Like cms-ber except it prints PEM encoded data, which is ASCII and
+   Like cms-raw except it prints PEM encoded data, which is ASCII and
    safe to print to terminals.
 cms-raw
    Print the payload of the CMS blob. This should be well-formed BER
-   encoded ASN.1 data.
+   encoded ASN.1 data. (This will print binary to stdout.)
 cms
    Print the ASN.1 decoded CMS data.
 code-directory-raw
@@ -94,10 +92,14 @@ code-directory-serialized
 code-directory-serialized-raw
    Reserialize the parsed code directory and emit its binary. Useful
    for comparing round-tripping of code directory data.
-linkededit-segment-raw
+linkedit-info
+   Information about the __LINKEDIT Mach-O segment in the binary.
+linkedit-segment-raw
    Complete content of the __LINKEDIT Mach-O segment as binary.
 macho-load-commands
    Print information about mach-o load commands in the binary.
+macho-segments
+   Print information about mach-o segments in the binary.
 requirements-raw
    Raw binary data composing the requirements blob/slot.
 requirements
@@ -112,9 +114,6 @@ requirements-serialized-raw
    Reserialize the code requirements blob and emit its binary.
 signature-raw
    Raw binary data composing the signature data embedded in the binary.
-segment-info
-   Information about Mach-O segments in the binary and where the
-   __LINKEDIT is in relationship to the binary.
 superblob
    The SuperBlob record and high-level details of embedded Blob
    records, including digests of every Blob.
@@ -309,7 +308,6 @@ can be specified via --timestamp-url. By default, Apple's server is used. The
 special value \"none\" can disable using a timestamp server.
 ";
 
-#[allow(unused)]
 const APPLE_TIMESTAMP_URL: &str = "http://timestamp.apple.com/ts01";
 
 const SUPPORTED_HASHES: &[&str; 6] = &[
@@ -380,6 +378,165 @@ fn command_compute_code_hashes(args: &ArgMatches) -> Result<(), AppleCodesignErr
     Ok(())
 }
 
+fn print_signed_data(
+    prefix: &str,
+    signed_data: &SignedData,
+    external_content: Option<Vec<u8>>,
+) -> Result<(), AppleCodesignError> {
+    println!(
+        "{}signed content (embedded): {:?}",
+        prefix,
+        signed_data.signed_content().map(hex::encode)
+    );
+    println!(
+        "{}signed content (external): {:?}... ({} bytes)",
+        prefix,
+        external_content.as_ref().map(|x| hex::encode(&x[0..40])),
+        external_content.as_ref().map(|x| x.len()).unwrap_or(0),
+    );
+
+    let content = if let Some(v) = signed_data.signed_content() {
+        Some(v)
+    } else {
+        external_content.as_ref().map(|v| v.as_ref())
+    };
+
+    if let Some(content) = content {
+        println!(
+            "{}signed content SHA-1:   {}",
+            prefix,
+            hex::encode(DigestType::Sha1.digest(&content)?)
+        );
+        println!(
+            "{}signed content SHA-256: {}",
+            prefix,
+            hex::encode(DigestType::Sha256.digest(&content)?)
+        );
+        println!(
+            "{}signed content SHA-384: {}",
+            prefix,
+            hex::encode(DigestType::Sha384.digest(&content)?)
+        );
+        println!(
+            "{}signed content SHA-512: {}",
+            prefix,
+            hex::encode(DigestType::Sha512.digest(&content)?)
+        );
+    }
+    println!(
+        "{}certificate count: {}",
+        prefix,
+        signed_data.certificates().count()
+    );
+    for (i, cert) in signed_data.certificates().enumerate() {
+        println!(
+            "{}certificate #{}: subject CN={}; self signed={}",
+            prefix,
+            i,
+            cert.subject_common_name()
+                .unwrap_or_else(|| "<unknown>".to_string()),
+            cert.subject_is_issuer()
+        );
+    }
+    println!("{}signer count: {}", prefix, signed_data.signers().count());
+    for (i, signer) in signed_data.signers().enumerate() {
+        println!(
+            "{}signer #{}: digest algorithm: {:?}",
+            prefix,
+            i,
+            signer.digest_algorithm()
+        );
+        println!(
+            "{}signer #{}: signature algorithm: {:?}",
+            prefix,
+            i,
+            signer.signature_algorithm()
+        );
+
+        if let Some(sa) = signer.signed_attributes() {
+            println!(
+                "{}signer #{}: content type: {}",
+                prefix,
+                i,
+                sa.content_type()
+            );
+            println!(
+                "{}signer #{}: message digest: {}",
+                prefix,
+                i,
+                hex::encode(sa.message_digest())
+            );
+            println!(
+                "{}signer #{}: signing time: {:?}",
+                prefix,
+                i,
+                sa.signing_time()
+            );
+        }
+
+        let digested_data = signer.signed_content_with_signed_data(&signed_data);
+
+        println!(
+            "{}signer #{}: signature content SHA-1:   {}",
+            prefix,
+            i,
+            hex::encode(DigestType::Sha1.digest(&digested_data)?)
+        );
+        println!(
+            "{}signer #{}: signature content SHA-256: {}",
+            prefix,
+            i,
+            hex::encode(DigestType::Sha256.digest(&digested_data)?)
+        );
+        println!(
+            "{}signer #{}: signature content SHA-384: {}",
+            prefix,
+            i,
+            hex::encode(DigestType::Sha384.digest(&digested_data)?)
+        );
+        println!(
+            "{}signer #{}: signature content SHA-512: {}",
+            prefix,
+            i,
+            hex::encode(DigestType::Sha512.digest(&digested_data)?)
+        );
+
+        if signed_data.signed_content().is_some() {
+            println!(
+                "{}signer #{}: digest valid: {}",
+                prefix,
+                i,
+                signer
+                    .verify_message_digest_with_signed_data(&signed_data)
+                    .is_ok()
+            );
+        }
+        println!(
+            "{}signer #{}: signature valid: {}",
+            prefix,
+            i,
+            signer
+                .verify_signature_with_signed_data(&signed_data)
+                .is_ok()
+        );
+
+        println!(
+            "{}signer #{}: time-stamp token present: {}",
+            prefix,
+            i,
+            signer.time_stamp_token_signed_data()?.is_some()
+        );
+
+        if let Some(tsp_signed_data) = signer.time_stamp_token_signed_data()? {
+            let prefix = format!("{}signer #{}: time-stamp token: ", prefix, i);
+
+            print_signed_data(&prefix, &tsp_signed_data, None)?;
+        }
+    }
+
+    Ok(())
+}
+
 fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
     let path = args
         .value_of("path")
@@ -394,27 +551,41 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
 
     let macho = get_macho_from_data(&data, index)?;
 
-    let sig = find_signature_data(&macho)?.ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
-
     match format {
         "blobs" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
 
             for blob in embedded.blobs {
                 let parsed = blob.into_parsed_blob()?;
                 println!("{:#?}", parsed);
             }
         }
-        "cms-ber" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+        "cms-info" => {
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
             if let Some(cms) = embedded.signature_data()? {
-                std::io::stdout().write_all(cms)?;
+                let signed_data = SignedData::parse_ber(cms)?;
+
+                let cd_data = if let Ok(Some(blob)) = embedded.code_directory() {
+                    Some(blob.to_blob_bytes()?)
+                } else {
+                    None
+                };
+
+                print_signed_data("", &signed_data, cd_data)?;
             } else {
                 eprintln!("no CMS data");
             }
         }
         "cms-pem" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
             if let Some(cms) = embedded.signature_data()? {
                 print!(
                     "{}",
@@ -428,7 +599,10 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             }
         }
         "cms-raw" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
             if let Some(cms) = embedded.signature_data()? {
                 std::io::stdout().write_all(cms)?;
             } else {
@@ -436,7 +610,10 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             }
         }
         "cms" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
             if let Some(cms) = embedded.signature_data()? {
                 let signed_data = SignedData::parse_ber(cms)?;
 
@@ -446,7 +623,9 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             }
         }
         "code-directory-raw" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
 
             if let Some(blob) = embedded.find_slot(CodeSigningSlot::CodeDirectory) {
                 std::io::stdout().write_all(blob.data)?;
@@ -455,7 +634,9 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             }
         }
         "code-directory-serialized-raw" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
 
             if let Ok(Some(cd)) = embedded.code_directory() {
                 std::io::stdout().write_all(&cd.to_blob_bytes()?)?;
@@ -464,7 +645,9 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             }
         }
         "code-directory-serialized" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
 
             if let Ok(Some(cd)) = embedded.code_directory() {
                 let serialized = cd.to_blob_bytes()?;
@@ -472,7 +655,9 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             }
         }
         "code-directory" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
 
             if let Some(cd) = embedded.code_directory()? {
                 println!("{:#?}", cd);
@@ -480,90 +665,9 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
                 eprintln!("no code directory");
             }
         }
-        "linkedit-segment-raw" => {
-            std::io::stdout().write_all(sig.linkedit_segment_data)?;
-        }
-        "macho-load-commands" => {
-            println!("load command count: {}", macho.load_commands.len());
-
-            for command in &macho.load_commands {
-                println!(
-                    "{}; offsets=0x{:x}-0x{:x} ({}-{}); size={}",
-                    goblin::mach::load_command::cmd_to_str(command.command.cmd()),
-                    command.offset,
-                    command.offset + command.command.cmdsize(),
-                    command.offset,
-                    command.offset + command.command.cmdsize(),
-                    command.command.cmdsize(),
-                );
-            }
-        }
-        "requirements-raw" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
-
-            if let Some(blob) = embedded.find_slot(CodeSigningSlot::RequirementSet) {
-                std::io::stdout().write_all(blob.data)?;
-            } else {
-                eprintln!("no requirements");
-            }
-        }
-        "requirements-rust" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
-
-            if let Some(reqs) = embedded.code_requirements()? {
-                for (typ, req) in &reqs.requirements {
-                    for expr in req.parse_expressions()?.iter() {
-                        println!("{} => {:#?}", typ, expr);
-                    }
-                }
-            } else {
-                eprintln!("no requirements");
-            }
-        }
-        "requirements-serialized-raw" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
-
-            if let Some(reqs) = embedded.code_requirements()? {
-                std::io::stdout().write_all(&reqs.to_blob_bytes()?)?;
-            } else {
-                eprintln!("no requirements");
-            }
-        }
-        "requirements-serialized" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
-
-            if let Some(reqs) = embedded.code_requirements()? {
-                let serialized = reqs.to_blob_bytes()?;
-                println!("{:#?}", RequirementSetBlob::from_blob_bytes(&serialized)?);
-            } else {
-                eprintln!("no requirements");
-            }
-        }
-        "requirements" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
-
-            if let Some(reqs) = embedded.code_requirements()? {
-                for (typ, req) in &reqs.requirements {
-                    for expr in req.parse_expressions()?.iter() {
-                        println!("{} => {}", typ, expr);
-                    }
-                }
-            } else {
-                eprintln!("no requirements");
-            }
-        }
-        "segment-info" => {
-            println!("segments count: {}", sig.segments_count);
-            for (i, segment) in macho.segments.iter().enumerate() {
-                println!(
-                    "segment #{}; {}; start offset {}; end offset {}; size {}",
-                    i,
-                    segment.name()?,
-                    segment.fileoff,
-                    segment.fileoff as usize + segment.data.len(),
-                    segment.data.len()
-                );
-            }
+        "linkedit-info" => {
+            let sig =
+                find_signature_data(&macho)?.ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
             println!("__LINKEDIT segment index: {}", sig.linkedit_segment_index);
             println!(
                 "__LINKEDIT segment start offset: {}",
@@ -595,11 +699,128 @@ fn command_extract(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             );
             println!("__LINKEDIT signature size: {}", sig.signature_data.len());
         }
+        "linkedit-segment-raw" => {
+            let sig =
+                find_signature_data(&macho)?.ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+            std::io::stdout().write_all(sig.linkedit_segment_data)?;
+        }
+        "macho-load-commands" => {
+            println!("load command count: {}", macho.load_commands.len());
+
+            for command in &macho.load_commands {
+                println!(
+                    "{}; offsets=0x{:x}-0x{:x} ({}-{}); size={}",
+                    goblin::mach::load_command::cmd_to_str(command.command.cmd()),
+                    command.offset,
+                    command.offset + command.command.cmdsize(),
+                    command.offset,
+                    command.offset + command.command.cmdsize(),
+                    command.command.cmdsize(),
+                );
+            }
+        }
+        "macho-segments" => {
+            println!("segments count: {}", macho.segments.len());
+            for (segment_index, segment) in macho.segments.iter().enumerate() {
+                let sections = segment.sections()?;
+
+                println!(
+                    "segment #{}; {}; offsets=0x{:x}-0x{:x}; size {}; section count {}",
+                    segment_index,
+                    segment.name()?,
+                    segment.fileoff,
+                    segment.fileoff as usize + segment.data.len(),
+                    segment.data.len(),
+                    sections.len()
+                );
+                for (section_index, (section, _)) in sections.into_iter().enumerate() {
+                    println!(
+                        "segment #{}; section #{}: {}; segment offsets=0x{:x}-0x{:x} size {}",
+                        segment_index,
+                        section_index,
+                        section.name()?,
+                        section.offset,
+                        section.offset as u64 + section.size,
+                        section.size
+                    );
+                }
+            }
+        }
+        "requirements-raw" => {
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
+            if let Some(blob) = embedded.find_slot(CodeSigningSlot::RequirementSet) {
+                std::io::stdout().write_all(blob.data)?;
+            } else {
+                eprintln!("no requirements");
+            }
+        }
+        "requirements-rust" => {
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
+            if let Some(reqs) = embedded.code_requirements()? {
+                for (typ, req) in &reqs.requirements {
+                    for expr in req.parse_expressions()?.iter() {
+                        println!("{} => {:#?}", typ, expr);
+                    }
+                }
+            } else {
+                eprintln!("no requirements");
+            }
+        }
+        "requirements-serialized-raw" => {
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
+            if let Some(reqs) = embedded.code_requirements()? {
+                std::io::stdout().write_all(&reqs.to_blob_bytes()?)?;
+            } else {
+                eprintln!("no requirements");
+            }
+        }
+        "requirements-serialized" => {
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
+            if let Some(reqs) = embedded.code_requirements()? {
+                let serialized = reqs.to_blob_bytes()?;
+                println!("{:#?}", RequirementSetBlob::from_blob_bytes(&serialized)?);
+            } else {
+                eprintln!("no requirements");
+            }
+        }
+        "requirements" => {
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+
+            if let Some(reqs) = embedded.code_requirements()? {
+                for (typ, req) in &reqs.requirements {
+                    for expr in req.parse_expressions()?.iter() {
+                        println!("{} => {}", typ, expr);
+                    }
+                }
+            } else {
+                eprintln!("no requirements");
+            }
+        }
         "signature-raw" => {
+            let sig =
+                find_signature_data(&macho)?.ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
             std::io::stdout().write_all(&sig.signature_data)?;
         }
         "superblob" => {
-            let embedded = parse_signature_data(&sig.signature_data)?;
+            let sig =
+                find_signature_data(&macho)?.ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
+            let embedded = macho
+                .code_signature()?
+                .ok_or(AppleCodesignError::BinaryNoCodeSignature)?;
 
             println!("file start offset: {}", sig.linkedit_signature_start_offset);
             println!("file end offset: {}", sig.linkedit_signature_end_offset);
@@ -886,7 +1107,6 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
             if timestamp_url != "none" {
                 warn!(&log, "using time-stamp protocol server {}", timestamp_url);
                 settings.set_time_stamp_url(timestamp_url)?;
-                error!(&log, "use of time-stamp tokens is known to break Apple signature verification; see https://github.com/indygreg/PyOxidizer/issues/399");
             }
         }
     }
@@ -990,6 +1210,30 @@ fn command_sign(args: &ArgMatches) -> Result<(), AppleCodesignError> {
         .expect("output_path presence should have been validated by clap");
 
     if input_path.is_file() {
+        if settings.binary_identifier(SettingsScope::Main).is_none() {
+            let identifier = input_path
+                .file_name()
+                .ok_or_else(|| {
+                    AppleCodesignError::CliGeneralError(
+                        "unable to resolve file name of binary".into(),
+                    )
+                })?
+                .to_string_lossy();
+
+            warn!(&log, "setting binary identifier to {}", identifier);
+            settings.set_binary_identifier(SettingsScope::Main, identifier);
+        }
+
+        if settings
+            .executable_segment_flags(SettingsScope::Main)
+            .is_none()
+        {
+            settings.set_executable_segment_flags(
+                SettingsScope::Main,
+                ExecutableSegmentFlags::MAIN_BINARY,
+            );
+        }
+
         warn!(&log, "signing {} as a Mach-O binary", input_path.display());
         let macho_data = std::fs::read(input_path)?;
 
@@ -1083,7 +1327,7 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                         .takes_value(true)
                         .possible_values(&[
                             "blobs",
-                            "cms-ber",
+                            "cms-info",
                             "cms-pem",
                             "cms-raw",
                             "cms",
@@ -1091,18 +1335,19 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                             "code-directory-serialized-raw",
                             "code-directory-serialized",
                             "code-directory",
+                            "linkedit-info",
                             "linkedit-segment-raw",
                             "macho-load-commands",
+                            "macho-segments",
                             "requirements-raw",
                             "requirements-rust",
                             "requirements-serialized-raw",
                             "requirements-serialized",
                             "requirements",
-                            "segment-info",
                             "signature-raw",
                             "superblob",
                         ])
-                        .default_value("segment-info")
+                        .default_value("linkedit-info")
                         .help("Which data to extract and how to format it"),
                 )
                 .arg(
@@ -1310,8 +1555,7 @@ fn main_impl() -> Result<(), AppleCodesignError> {
                     Arg::with_name("timestamp_url")
                         .long("timestamp-url")
                         .takes_value(true)
-                        // TODO enable once https://github.com/indygreg/PyOxidizer/issues/399 is fixed.
-                        //.default_value(APPLE_TIMESTAMP_URL)
+                        .default_value(APPLE_TIMESTAMP_URL)
                         .help(
                             "URL of timestamp server to use to obtain a token of the CMS signature",
                         ),
@@ -1360,7 +1604,7 @@ fn main() {
     let exit_code = match main_impl() {
         Ok(()) => 0,
         Err(err) => {
-            eprintln!("Error: {:?}", err);
+            eprintln!("Error: {}", err);
             1
         }
     };
