@@ -27,8 +27,7 @@ use {
         code_hash::compute_code_hashes,
         error::AppleCodesignError,
         macho::{
-            find_signature_data, parse_signature_data, CodeSigningSlot, DigestType,
-            EmbeddedSignature,
+            find_signature_data, AppleSignable, CodeSigningSlot, DigestType, EmbeddedSignature,
         },
     },
     cryptographic_message_syntax::{CmsError, SignedData},
@@ -58,6 +57,7 @@ pub enum VerificationProblemType {
     SignatureNotLastLinkeditData,
     NoCryptographicSignature,
     CmsError(CmsError),
+    CmsOldDigestAlgorithm(DigestAlgorithm),
     CmsOldSignatureAlgorithm(SignatureAlgorithm),
     NoCodeDirectory,
     CodeDirectoryOldDigestAlgorithm(DigestType),
@@ -105,6 +105,9 @@ impl std::fmt::Display for VerificationProblem {
                 "no cryptographic signature present".to_string()
             }
             VerificationProblemType::CmsError(e) => format!("CMS error: {}", e),
+            VerificationProblemType::CmsOldDigestAlgorithm(alg) => {
+                format!("insecure digest algorithm used: {:?}", alg)
+            }
             VerificationProblemType::CmsOldSignatureAlgorithm(alg) => {
                 format!("insecure signature algorithm used: {:?}", alg)
             }
@@ -292,8 +295,11 @@ fn verify_macho_internal(macho: &MachO, context: VerificationContext) -> Vec<Ver
         });
     }
 
-    let signature = match parse_signature_data(&signature_data.signature_data) {
-        Ok(embedded) => embedded,
+    let signature = match macho.code_signature() {
+        Ok(Some(signature)) => signature,
+        Ok(None) => {
+            panic!("no signature should have been handled above");
+        }
         Err(e) => {
             problems.push(VerificationProblem {
                 context,
@@ -357,18 +363,27 @@ fn verify_cms_signature(data: &[u8], context: VerificationContext) -> Vec<Verifi
 
     for signer in signed_data.signers() {
         match signer.digest_algorithm() {
+            DigestAlgorithm::Sha1 => {
+                problems.push(VerificationProblem {
+                    context: context.clone(),
+                    problem: VerificationProblemType::CmsOldDigestAlgorithm(
+                        signer.digest_algorithm(),
+                    ),
+                });
+            }
+            DigestAlgorithm::Sha384 => {}
             DigestAlgorithm::Sha256 => {}
             DigestAlgorithm::Sha512 => {}
         }
 
         match signer.signature_algorithm() {
-            SignatureAlgorithm::Sha256Rsa
-            | SignatureAlgorithm::Sha512Rsa
+            SignatureAlgorithm::RsaSha256
+            | SignatureAlgorithm::RsaSha384
+            | SignatureAlgorithm::RsaSha512
             | SignatureAlgorithm::EcdsaSha256
             | SignatureAlgorithm::EcdsaSha384
             | SignatureAlgorithm::Ed25519 => {}
-            // RsaesPkcsV15 appears to be in widespread use. Should we still notify?
-            SignatureAlgorithm::RsaesPkcsV15 | SignatureAlgorithm::Sha1Rsa => {
+            SignatureAlgorithm::RsaSha1 => {
                 problems.push(VerificationProblem {
                     context: context.clone(),
                     problem: VerificationProblemType::CmsOldSignatureAlgorithm(
